/*
* anonymoX client
* 	for Firefox
* 
* Author 	Nils Hügelmann	<nils@anonymox.net>
* Author 	Florian Prott	<florian@anonymox.net>
* Author 	Christian Otto	<chris@anonymox.net>
*
* www.anonymox.net
* 
* License: CC BY-NC-ND 3.0, see LICENSE.TXT or http://creativecommons.org/licenses/by-nc-nd/3.0/
* 
* Use at your own risk
* 
* This is the released, uncommented version of the code.
*/


"use strict";

var anonymox = {};

Components.utils.import("resource://anonymox/components.jsm", anonymox);

anonymox.overlay = {
		
	
	installButton : function(toolbarId, id, afterId) {
	    if (!document.getElementById(id)) {
	        var toolbar = document.getElementById(toolbarId);

	        
	        var before = null;
	        if (afterId) {
	            let elem = document.getElementById(afterId);
	            if (elem && elem.parentNode == toolbar)
	                before = elem.nextElementSibling;
	        }

	        toolbar.insertItem(id, before);
	        toolbar.setAttribute("currentset", toolbar.currentSet);
	        document.persist(toolbar.id, "currentset");

	        if (toolbarId == "addon-bar")
	            toolbar.collapsed = false;
	    }
	},

	buildStatusBar : function() {
		let info = new anonymox.Info("overlay.buildStatusBar");
		Cu.import("resource:///modules/CustomizableUI.jsm");
		let XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

		CustomizableUI.createWidget({
			id : "anonymox-view-button",
			type: "custom",
			defaultArea : CustomizableUI.AREA_NAVBAR,
			onBuild : function(document) {
				let node = document.createElementNS(XUL_NS, "toolbaritem");
				node.setAttribute("id", "anonymox-view-button");
				node.setAttribute("removable", "true");
				node.setAttribute("label", "AnonymoX");
				node.setAttribute("title", "AnonymoX");

				let btnNode = document.createElementNS(XUL_NS, "toolbarbutton");
				btnNode.setAttribute("id", "anonymox-view-icon");
				btnNode.setAttribute("tooltip", "anonymox-infopanel");

				btnNode.setAttribute("class", "toolbarbutton-1");
				btnNode.setAttribute("label", "AnonymoX");

				btnNode.addEventListener("click", function(e) {
					e.target.ownerDocument.defaultView.anonymox.panel.open('viewButton');
				});

				let hBox = document.createElementNS(XUL_NS, "hbox");
				hBox.setAttribute("id", "anonymox-statusinfo");
				hBox.setAttribute("align", "center");

				node.appendChild(btnNode);
				node.appendChild(hBox);

				let profileLabel = document.createElementNS(XUL_NS, "label");
				profileLabel.setAttribute("id", "anonymox-statusLabel");
				profileLabel.setAttribute("style", "margin-left: 0px; padding-left: 2px;");
				profileLabel.addEventListener("click", function(e) {
					e.target.ownerDocument.defaultView.anonymox.panel.open('viewButton');
				});
				hBox.appendChild(profileLabel);

				let externalFlag = document.createElementNS(XUL_NS, "image");
				externalFlag.setAttribute("id", "anonymox-externalFlag");
				externalFlag.setAttribute("src", "chrome://");
				anonymox.utils.jQuery(externalFlag).hide();
				hBox.appendChild(externalFlag);

				let externalIp = document.createElementNS(XUL_NS, "label");
				externalIp.setAttribute("id", "anonymox-externalIp");
				anonymox.utils.jQuery(externalIp).hide();
				hBox.appendChild(externalIp);

				externalFlag.addEventListener("click", function() {
					anonymox.ui.refreshExternalInfo(window);
				});

				externalIp.addEventListener("click", function() {
					anonymox.ui.refreshExternalInfo(window);
				});

				let listener = {
					onWidgetAdded : function(widgetId, area, pos) {
						info.debug("widget added: " + widgetId + " " + area + " " + pos);
						anonymox.ui.updateUIElements();
						anonymox.ui.updateStatusLabels();
						anonymox.ui.safeWindowIterator(function(win) {
							anonymox.ui.updateExternalInfo(win);
						});
					},
					onWidgetRemoved : function(widgetId, prevArea) {
						info.debug("widget removed: " + widgetId + " " + prevArea);
					}.bind(this),
					onWidgetMoved : function(widgetId, area) {
						info.debug("widget moved: " + widgetId + " " + area);
					}.bind(this),
					onWidgetReset : function(widgetId) {
						info.debug("widget reset: " + widgetId);
					}.bind(this),
				}

				CustomizableUI.addListener(listener);

				return node;
			},
			onViewShowing : function() {
			},
			onCommand : function() {
			},
			onClick : function() {
			}
		});
	},

	
	progressListener : {
		QueryInterface : function(aIID) {
			if (aIID.equals(Ci.nsIWebProgressListener)
					|| aIID.equals(Ci.nsISupportsWeakReference)
					|| aIID.equals(Ci.nsISupports))
				return this;
			throw Components.results.NS_NOINTERFACE;
		},
		
		onLocationChange: function(aWebProgress, aRequest, aURI) {
			let info = new anonymox.Info("overlay.progressListener.onLocationChange");
			try {
				let browser = gBrowser.getBrowserForDocument(aWebProgress.DOMWindow.document);
				info.verbose("spec: " + aURI.asciiSpec + " DOMWindow " + aWebProgress.DOMWindow + " browser " + browser);
				if (browser) {
					anonymox.ui.locationOrProfileChange(window, browser, aURI.asciiSpec);
				} else {
					info.debug("ABORT: No browser found for " + aURI);
				}
			} catch(e) {
				info.error(e);
			}
		},
		
		onStateChange : function(aWebProgress, aRequest, aStateFlags, aStatus) {
			let info = new anonymox.Info("overlay.progressListener.onStateChange");
			try {
				info.verbose(new Date().getTime() + " " + aStateFlags.toString(16) + "|" + aStatus.toString(16));
				
				if (aStateFlags & Ci.nsIWebProgressListener.STATE_START) {
					
					if (aRequest) {
						
						if (aRequest instanceof Ci.nsIChannel || "URI" in aRequest) {
		
							info.verbose("Started: " + aRequest.URI.asciiSpec);
							
							let browser = gBrowser.getBrowserForDocument(aWebProgress.DOMWindow.document);
							if (browser) {
								anonymox.ui.locationOrProfileChange(window, browser, aRequest.URI.asciiSpec);
							} else {
								info.verbose("ABORT: No browser found for " + aRequest.URI.asciiSpec);
							}
						} else {
							info.verbose("ABORT: Cannot get URI from aRequest");
						}
					} else {
						info.verbose("ABORT: No aRequest");
					}
				} else if (aStateFlags & Ci.nsIWebProgressListener.STATE_STOP) {
					
					
		            
					
					info.verbose("STATE_STOP status " + aStatus);
	
					if (aStatus == Components.results.NS_ERROR_PROXY_CONNECTION_REFUSED) {
						info.warning("Detected possible Proxy-Problem.(NS_ERROR_PROXY_CONNECTION_REFUSED)");
						
					}
					
					try {
						
						let browser = gBrowser.getBrowserForDocument(aWebProgress.DOMWindow.document);
						if (browser) {
							anonymox.ui.locationOrProfileChange(window, browser);
						} else {
							info.verbose("ABORT: No browser found");
						}
					} catch(e) {
						
					}
				}
			} catch(e) {
				info.error(e);
			}
		}
	}
};



window.addEventListener("load", function() {
	if (!anonymox.state.isLoaded)
		anonymox.core.startUp(window);
	else
		anonymox.ui.newWindowOpened(window);
}, false);
document.addEventListener("anonymox_websiteComm", function(e) { anonymox.core.websiteCommEventHandler(e); }, false, true);
